@@ -1,20 +1,31 @@
 import os
-import base64
+import io
 import hashlib
 import requests
+import tempfile
 import mimetypes
+import numpy as np
 
+from PIL import Image
 from fnmatch import fnmatch
 from datetime import datetime
-from io import BytesIO
 from . import config
 
 
+API_URL = 'https://api.abraia.me'
+tempdir = tempfile.gettempdir()
+
+
+def file_path(f, userid):
+    return f['source'][len(userid)+1:]
+
+
 def md5sum(src):
     hash_md5 = hashlib.md5()
-    with open(src, 'rb') as f:
-        for chunk in iter(lambda: f.read(4096), b''):
-            hash_md5.update(chunk)
+    f = io.BytesIO(src.getvalue()) if isinstance(src, io.BytesIO) else open(src, 'rb')
+    for chunk in iter(lambda: f.read(4096), b''):
+        hash_md5.update(chunk)
+    f.close()
     return hash_md5.hexdigest()
 
 
@@ -28,13 +39,15 @@ class APIError(Exception):
             self.message = ''
 
 
-class Client(object):
-    def __init__(self):
+class Abraia:
+    def __init__(self, folder=''):
         self.auth = config.load_auth()
+        self.userid = self.load_user().get('id')
+        self.folder = folder
 
     def load_user(self):
         if self.auth[0] and self.auth[1]:
-            url = '{}/users'.format(config.API_URL)
+            url = f"{API_URL}/users"
             resp = requests.get(url, auth=self.auth)
             if resp.status_code != 200:
                 raise APIError(resp.text, resp.status_code)
@@ -42,157 +55,152 @@ class Client(object):
         return {}
 
     def list_files(self, path=''):
-        url = '{}/files/{}'.format(config.API_URL, path)
+        dirname = os.path.dirname(path)
+        basename = os.path.basename(path)
+        folder = dirname + '/' if dirname else dirname
+        url = f"{API_URL}/files/{self.userid}/{folder}"
         resp = requests.get(url, auth=self.auth)
         if resp.status_code != 200:
             raise APIError(resp.text, resp.status_code)
         resp = resp.json()
         for f in resp['files']:
             f['date'] = datetime.fromtimestamp(f['date'])
-        return resp['files'], resp['folders']
+        files, folders = resp['files'], resp['folders']
+        files = list(map(lambda f: {'path': file_path(f, self.userid), 'name': f['name'], 'size': f['size'], 'date': f['date']}, files))
+        folders = list(map(lambda f: {'path': file_path(f, self.userid), 'name': f['name']}, folders))
+        if basename:
+            files = list(filter(lambda f: fnmatch(f['path'], path), files))
+            folders = list(filter(lambda f: fnmatch(f['path'], path), folders))
+        return files, folders
 
     def upload_remote(self, url, path):
         json = {'url': url}
-        url = '{}/files/{}'.format(config.API_URL, path)
+        url = f"{API_URL}/files/{self.userid}/{path}"
         resp = requests.post(url, json=json, auth=self.auth)
         if resp.status_code != 201:
             raise APIError(resp.text, resp.status_code)
         resp = resp.json()
-        return resp['file']
+        return file_path(resp['file'], self.userid)
 
-    def upload_file(self, file, path):
-        source = path + os.path.basename(file) if path.endswith('/') else path
-        name = os.path.basename(source)
-        md5 = md5sum(file) # TODO: Refactor names to src, dest, path (cloud)
+    def upload_file(self, src, path=''):
+        if isinstance(src, str) and src.startswith('http'):
+            return self.upload_remote(src, path)
+        if path == '' or path.endswith('/'):
+            path = path + os.path.basename(src)
+        name = os.path.basename(path)
+        md5 = md5sum(src) # TODO: Refactor names to src, dest, path (cloud)
         type = mimetypes.guess_type(name)[0] or 'binary/octet-stream'
-        json = {'name': name, 'type': type, 'md5': md5}  if md5 else {'name': name, 'type': type}
-        url = '{}/files/{}'.format(config.API_URL, source)
-        resp = requests.post(url, json=json, auth=self.auth) # TODO: Refactor to reduce requests code (like with client.js)
+        json = {'name': name, 'type': type, 'md5': md5} if md5 else {'name': name, 'type': type}
+        url = f"{API_URL}/files/{self.userid}/{path}"
+        print('>', url, json)
+        resp = requests.post(url, json=json, auth=self.auth)
         if resp.status_code != 201:
             raise APIError(resp.text, resp.status_code)
         resp = resp.json()
         url = resp.get('uploadURL')
         if url:
-            data = file if isinstance(file, BytesIO) else open(file, 'rb')
+            data = src if isinstance(src, io.BytesIO) else open(src, 'rb')
             resp = requests.put(url, data=data, headers={'Content-Type': type})
             if resp.status_code != 200:
                 raise APIError(resp.text, resp.status_code)
-        return {'name': name, 'source': source}
+        return file_path({'name': name, 'source': f"{self.userid}/{path}"}, self.userid)
 
     def move_file(self, old_path, new_path):
-        url = '{}/files/{}'.format(config.API_URL, new_path)
-        resp = requests.post(url, json={'store': old_path}, auth=self.auth)
+        json = {'store': f"{self.userid}/{old_path}"}
+        url = f"{API_URL}/files/{self.userid}/{new_path}"
+        resp = requests.post(url, json=json, auth=self.auth)
         if resp.status_code != 201:
             raise APIError(resp.text, resp.status_code)
         resp = resp.json()
-        return resp['file']
+        return file_path(resp['file'], self.userid)
 
     def download_file(self, path, dest=''):
-        # T0D0 Add dest ptina paramater to save file
-        url = '{}/files/{}'.format(config.API_URL, path)
+        url = f"{API_URL}/files/{self.userid}/{path}"
         resp = requests.get(url, stream=True, auth=self.auth)
         if resp.status_code != 200:
             raise APIError(resp.text, resp.status_code)
-        return BytesIO(resp.content)
+        if dest:
+            with open(dest, 'wb') as f:
+                f.write(resp.content)
+            return dest
+        return io.BytesIO(resp.content)
 
     def remove_file(self, path):
-        url = '{}/files/{}'.format(config.API_URL, path)
+        url = f"{API_URL}/files/{self.userid}/{path}"
         resp = requests.delete(url, auth=self.auth)
         if resp.status_code != 200:
             raise APIError(resp.text, resp.status_code)
         resp = resp.json()
-        return resp['file']
+        return file_path(resp['file'], self.userid)
 
     def load_metadata(self, path):
-        url = '{}/metadata/{}'.format(config.API_URL, path)
+        url = f"{API_URL}/metadata/{self.userid}/{path}"
         resp = requests.get(url, auth=self.auth)
         if resp.status_code != 200:
             raise APIError(resp.text, resp.status_code)
         return resp.json()
 
     def remove_metadata(self, path):
-        url = '{}/metadata/{}'.format(config.API_URL, path)
+        url = f"{API_URL}/metadata/{self.userid}/{path}"
         resp = requests.delete(url, auth=self.auth)
         if resp.status_code != 200:
             raise APIError(resp.text, resp.status_code)
         return resp.json()
 
-    # def analyze_image(self, path, params={}):
-    #     url = '{}/analysis/{}'.format(config.API_URL, path)
-    #     resp = requests.get(url, auth=self.auth)
-    #     if resp.status_code != 200:
-    #         raise APIError(resp.text, resp.status_code)
-    #     resp = resp.json()
-    #     if resp.get('salmap'):
-    #         resp['salmap'] = BytesIO(base64.b64decode(resp['salmap'][23:]))
-    #     return resp
-
-    def detect_labels(self, path, params={}):
-        url = '{}/rekognition/{}'.format(config.API_URL, path)
-        resp = requests.get(url, params=params, auth=self.auth)
-        if resp.status_code != 200:
-            raise APIError(resp.text, resp.status_code)
-        return resp.json()
-
-    def transform_image(self, path, params={}):
+    def transform_image(self, path, dest, params={'quality': 'auto'}):
+        ext = dest.split('.').pop().lower()
+        params['format'] = params.get('format') or ext
         if params.get('action'):
-            params['background'] = '{}/images/{}'.format(config.API_URL, path)
+            params['background'] = f"{API_URL}/images/{self.userid}/{path}"
             if params.get('fmt') is None:
                 params['fmt'] = params['background'].split('.').pop()
-            path = '{}/{}'.format(path.split('/')[0], params['action'])
-        url = '{}/images/{}'.format(config.API_URL, path)
+            path = f"{self.userid}/{params['action']}"
+        url = f"{API_URL}/images/{self.userid}/{path}"
         resp = requests.get(url, params=params, stream=True, auth=self.auth)
         if resp.status_code != 200:
             raise APIError(resp.text, resp.status_code)
-        return BytesIO(resp.content)
-
-
-class Abraia(Client):
-    def __init__(self, folder=''):
-        super(Abraia, self).__init__()
-        self.userid = self.load_user().get('id')
-        self.folder = folder
-        self.params = {}
-        self.path = ''
-
-    def list(self, path=''):
-        length = len(self.userid) + 1
-        dirname = os.path.dirname(path)
-        basename = os.path.basename(path)
-        folder = dirname + '/' if dirname else dirname
-        # TODO: Change path to manage userid
-        files, folders = self.list_files(path=self.userid + '/' + folder)
-        files = list(map(lambda f: {'path': f['source'][length:], 'name': f['name'], 'size': f['size'], 'date': f['date']}, files))
-        folders = list(map(lambda f: {'path': f['source'][length:], 'name': f['name']}, folders))
-        if basename:
-            files = list(filter(lambda f: fnmatch(f['path'], path), files))
-            folders = list(filter(lambda f: fnmatch(f['path'], path), folders))
-        return files, folders
-
-    def upload(self, src, path=''):
-        length = len(self.userid) + 1
-        if isinstance(src, str) and src.startswith('http'):
-            f = self.upload_remote(src, self.userid + '/')
-            return {'path': f['source'][length:]}
-        f = self.upload_file(src, self.userid + '/' + path)
-        return {'path': f['source'][length:]}
-
-    def download(self, path, dest=''):
-        buffer = self.download_file(self.userid + '/' + path)
-        if dest:
-            with open(dest, 'wb') as f:
-                f.write(buffer.getbuffer())
-            return dest
-        return buffer
-
-    def remove(self, path):
-        length = len(self.userid) + 1
-        f = self.remove_file(self.userid + '/' + path)
-        return {'path': f['source'][length:]}
-
-    def transform(self, path, dest, params={'quality': 'auto'}):
-        ext = dest.split('.').pop().lower()
-        params['format'] = self.params.get('format') or ext
-        buffer = self.transform_image(self.userid + '/' + path, params=params)
         with open(dest, 'wb') as f:
-            f.write(buffer.getvalue())
+            f.write(resp.content)
+
+    def load_file(self, path):
+        stream = self.download_file(path)
+        try:
+            return stream.getvalue().decode('utf-8')
+        except:
+            return stream
+
+    def load_image(self, path):
+        stream = self.download_file(path)
+        return np.asarray(Image.open(stream))
+
+    def save_file(self, path, stream):
+        stream =  io.BytesIO(bytes(stream, 'utf-8')) if isinstance(stream, str) else stream
+        return self.upload_file(stream, path)
+
+    def save_image(self, path, img):
+        # stream = io.BytesIO()
+        # mime = mimetypes.guess_type(path)[0]
+        # format = mime.split('/')[1]
+        # Image.fromarray(img).save(stream, format)
+        # print(mime, format)
+        basename = os.path.basename(path)
+        src = os.path.join(tempdir, basename)
+        Image.fromarray(img).save(src)
+        return self.upload_file(src, path)
+
+    def capture_text(self, path):
+        url = f"{API_URL}/rekognition/{self.userid}/{path}"
+        resp = requests.get(url, params={'mode': 'text'}, auth=self.auth)
+        if resp.status_code != 200:
+            raise APIError(resp.text, resp.status_code)
+        text = list(filter(lambda t: t.get('ParentId') is None, resp.json().get('Text')));
+        return [t.get('DetectedText') for t in text]
+
+    def detect_labels(self, path):
+        url = f"{API_URL}/rekognition/{self.userid}/{path}"
+        resp = requests.get(url, params={'mode': 'labels'}, auth=self.auth)
+        if resp.status_code != 200:
+            raise APIError(resp.text, resp.status_code)
+        labels = resp.json().get('Labels')
+        return [l.get('Name') for l in labels]
+    
\ No newline at end of file
